@@ -1,8 +1,10 @@
+import json
 import logging
 import os
 import re
 import shutil
 import sys
+import traceback
 from collections import namedtuple
 from contextlib import contextmanager
 from copy import deepcopy
@@ -15,11 +17,17 @@ from typing import Dict, List, Optional, Tuple, Union
 
 import requests
 from colorama import Fore, Style
-from progressbar import ProgressBar
 from requests import HTTPError
 
 from grayskull.base.base_recipe import AbstractRecipeModel
-from grayskull.cli import WIDGET_BAR_DOWNLOAD
+from grayskull.base.pkg_info import is_pkg_available
+from grayskull.base.track_packages import solve_list_pkg_name
+from grayskull.cli.stdout import (
+    manage_progressbar,
+    print_msg,
+    print_requirements,
+    progressbar_with_status,
+)
 from grayskull.license.discovery import ShortLicense, search_license_file
 from grayskull.utils import get_vendored_dependencies
 
@@ -34,9 +42,15 @@ class PyPi(AbstractRecipeModel):
     PKG_NEEDS_CXX_COMPILER = ("pybind11",)
     RE_DEPS_NAME = re.compile(r"^\s*([\.a-zA-Z0-9_-]+)", re.MULTILINE)
     PIN_PKG_COMPILER = {"numpy": "<{ pin_compatible('numpy') }}"}
+    PYPI_CONFIG = Path(os.path.dirname(__file__)) / "config.yaml"
 
-    def __init__(self, name=None, version=None, force_setup=False):
-        self._force_setup = force_setup
+    def __init__(
+        self,
+        name: Optional[str] = None,
+        version: Optional[str] = None,
+        download: bool = False,
+    ):
+        self._download = download
         self._setup_metadata = None
         self._is_arch = False
         super(PyPi, self).__init__(name=name, version=version)
@@ -50,7 +64,7 @@ class PyPi(AbstractRecipeModel):
         :param dest: Folder were the method will download the sdist
         """
         name = sdist_url.split("/")[-1]
-        print(
+        print_msg(
             f"{Fore.GREEN}Starting the download of the sdist package"
             f" {Fore.BLUE}{Style.BRIGHT}{name}"
         )
@@ -58,9 +72,9 @@ class PyPi(AbstractRecipeModel):
         response = requests.get(sdist_url, allow_redirects=True, stream=True, timeout=5)
         total_size = int(response.headers["Content-length"])
 
-        with ProgressBar(
-            widgets=WIDGET_BAR_DOWNLOAD, max_value=total_size, prefix=f"{name} "
-        ) as bar, open(dest, "wb") as pkg_file:
+        with manage_progressbar(max_value=total_size, prefix=f"{name} ") as bar, open(
+            dest, "wb"
+        ) as pkg_file:
             progress_val = 0
             chunk_size = 512
             for chunk_data in response.iter_content(chunk_size=chunk_size):
@@ -69,9 +83,8 @@ class PyPi(AbstractRecipeModel):
                     progress_val += chunk_size
                     bar.update(min(progress_val, total_size))
 
-    @staticmethod
     @lru_cache(maxsize=10)
-    def _get_sdist_metadata(sdist_url: str, name: str) -> dict:
+    def _get_sdist_metadata(self, sdist_url: str, name: str) -> dict:
         """Method responsible to return the sdist metadata which is basically
         the metadata present in setup.py and setup.cfg
 
@@ -84,9 +97,11 @@ class PyPi(AbstractRecipeModel):
         path_pkg = os.path.join(temp_folder, pkg_name)
 
         PyPi._download_sdist_pkg(sdist_url=sdist_url, dest=path_pkg)
+        if self._download:
+            self.files_to_copy.append(path_pkg)
         log.debug(f"Unpacking {path_pkg} to {temp_folder}")
         shutil.unpack_archive(path_pkg, temp_folder)
-        print(f"{Fore.LIGHTBLACK_EX}Recovering information from setup.py")
+        print_msg("Recovering information from setup.py")
         with PyPi._injection_distutils(temp_folder) as metadata:
             metadata["sdist_path"] = temp_folder
             return metadata
@@ -117,13 +132,30 @@ class PyPi(AbstractRecipeModel):
         if "extras_require" in result:
             result["extras_require"] = get_full_list("extras_require")
         if "setup_requires" in result:
+            result["setup_requires"] = get_full_list("setup_requires")
             if "setuptools-scm" in result["setup_requires"]:
                 result["setup_requires"].remove("setuptools-scm")
-            result["setup_requires"] = get_full_list("setup_requires")
         if "compilers" in result:
             result["compilers"] = get_full_list("compilers")
         return result
 
+    @staticmethod
+    def __rm_duplicated_deps(
+        all_requirements: Union[list, set, None]
+    ) -> Optional[list]:
+        if not all_requirements:
+            return None
+        new_value = []
+        for dep in all_requirements:
+            if (
+                dep in new_value
+                or dep.replace("-", "_") in new_value
+                or dep.replace("_", "-") in new_value
+            ):
+                continue
+            new_value.append(dep)
+        return new_value
+
     @staticmethod
     def _get_setup_cfg(source_path: str) -> dict:
         """Method responsible to extract the setup.cfg metadata
@@ -135,7 +167,7 @@ class PyPi(AbstractRecipeModel):
         from setuptools.config import read_configuration
 
         log.debug(f"Started setup.cfg from {source_path}")
-        print(f"{Fore.LIGHTBLACK_EX}Recovering metadata from setup.cfg")
+        print_msg("Recovering metadata from setup.cfg")
         path_setup_cfg = list(Path(source_path).rglob("setup.cfg"))
         if not path_setup_cfg:
             return {}
@@ -173,12 +205,14 @@ class PyPi(AbstractRecipeModel):
 
         setup_core_original = core.setup
         old_dir = os.getcwd()
-        path_setup = list(Path(folder).rglob("setup.py"))[0]
+        path_setup = search_setup_root(folder)
         os.chdir(os.path.dirname(str(path_setup)))
 
         data_dist = {}
 
         def __fake_distutils_setup(*args, **kwargs):
+            if not isinstance(kwargs, dict) or not kwargs:
+                return
             data_dist.update(kwargs)
             if not data_dist.get("setup_requires"):
                 data_dist["setup_requires"] = []
@@ -186,7 +220,7 @@ class PyPi(AbstractRecipeModel):
                 kwargs.get("setup_requires") if kwargs.get("setup_requires") else []
             )
 
-            if "use_scm_version" in kwargs and kwargs["use_scm_version"]:
+            if "use_scm_version" in data_dist and kwargs["use_scm_version"]:
                 log.debug("setuptools_scm found on setup.py")
                 if "setuptools_scm" not in data_dist["setup_requires"]:
                     data_dist["setup_requires"] += ["setuptools_scm"]
@@ -212,23 +246,29 @@ class PyPi(AbstractRecipeModel):
         try:
             core.setup = __fake_distutils_setup
             path_setup = str(path_setup)
-            print(f"{Fore.LIGHTBLACK_EX}Executing injected distutils...")
+            print_msg("Executing injected distutils...")
             PyPi.__run_setup_py(path_setup, data_dist)
             if not data_dist or not data_dist.get("install_requires", None):
-                print(
-                    f"{Fore.LIGHTBLACK_EX}No data was recovered from setup.py."
-                    f" Forcing to execute the setup.py as script"
+                print_msg(
+                    "No data was recovered from setup.py."
+                    " Forcing to execute the setup.py as script"
                 )
                 PyPi.__run_setup_py(path_setup, data_dist, run_py=True)
             yield data_dist
-        except Exception as err:  # noqa
-            log.debug(f"Exception occurred when executing sdist injection: err {err}")
+        except:  # noqa
+            log.debug(
+                "Exception occurred when executing sdist injection:"
+                f" {traceback.format_exc()}"
+            )
             yield data_dist
-        core.setup = setup_core_original
-        os.chdir(old_dir)
+        finally:
+            core.setup = setup_core_original
+            os.chdir(old_dir)
 
     @staticmethod
-    def __run_setup_py(path_setup: str, data_dist: dict, run_py=False):
+    def __run_setup_py(
+        path_setup: str, data_dist: dict, run_py=False, deps_installed=None
+    ):
         """Method responsible to run the setup.py
 
         :param path_setup: full path to the setup.py
@@ -236,6 +276,7 @@ class PyPi(AbstractRecipeModel):
         :param run_py: If it should run the setup.py with run_py, otherwise it will run
         invoking the distutils directly
         """
+        deps_installed = deps_installed if deps_installed else []
         original_path = deepcopy(sys.path)
         pip_dir = mkdtemp(prefix="pip-dir-")
         if not os.path.exists(pip_dir):
@@ -259,11 +300,17 @@ class PyPi(AbstractRecipeModel):
                 f"When executing setup.py did not find the module: {err.name}."
                 f" Exception: {err}"
             )
-            PyPi._pip_install_dep(data_dist, err.name, pip_dir)
-            PyPi.__run_setup_py(path_setup, data_dist, run_py)
-        except Exception as err:  # noqa
+            dep_install = err.name
+            if dep_install in deps_installed:
+                dep_install = dep_install.split(".")[0]
+            if dep_install not in deps_installed:
+                deps_installed.append(dep_install)
+                PyPi._pip_install_dep(data_dist, dep_install, pip_dir)
+                PyPi.__run_setup_py(
+                    path_setup, data_dist, run_py, deps_installed=deps_installed
+                )
+        except Exception as err:
             log.debug(f"Exception when executing setup.py as script: {err}")
-            pass
         data_dist.update(
             PyPi._merge_sdist_metadata(
                 data_dist, PyPi._get_setup_cfg(os.path.dirname(str(path_setup)))
@@ -298,12 +345,20 @@ class PyPi(AbstractRecipeModel):
             data_dist["setup_requires"] = []
         if dep_name == "pkg_resources":
             dep_name = "setuptools"
-        if (
-            dep_name.lower() not in data_dist["setup_requires"]
-            and dep_name.lower() != "setuptools"
-        ):
-            data_dist["setup_requires"].append(dep_name.lower())
-        check_output(["pip", "install", dep_name, f"--target={pip_dir}"])
+        try:
+            check_output(["pip", "install", dep_name, f"--target={pip_dir}"])
+        except Exception as err:
+            log.error(
+                f"It was not possible to install {dep_name}.\n"
+                f"Command: pip install {dep_name} --target={pip_dir}.\n"
+                f"Error: {err}"
+            )
+        else:
+            if (
+                dep_name.lower() not in data_dist["setup_requires"]
+                and dep_name.lower() != "setuptools"
+            ):
+                data_dist["setup_requires"].append(dep_name.lower())
 
     @staticmethod
     def _merge_pypi_sdist_metadata(pypi_metadata: dict, sdist_metadata: dict) -> dict:
@@ -325,12 +380,13 @@ class PyPi(AbstractRecipeModel):
             "version": get_val("version"),
             "source": pypi_metadata.get("source"),
             "packages": get_val("packages"),
-            "home": get_val("home"),
+            "url": get_val("url"),
             "classifiers": get_val("classifiers"),
             "compilers": PyPi._get_compilers(requires_dist, sdist_metadata),
             "entry_points": PyPi._get_entry_points_from_sdist(sdist_metadata),
             "summary": get_val("summary"),
-            "requires_python": get_val("requires_python"),
+            "requires_python": pypi_metadata.get("requires_python")
+            or sdist_metadata.get("python_requires"),
             "doc_url": get_val("doc_url"),
             "dev_url": get_val("dev_url"),
             "license": get_val("license"),
@@ -367,14 +423,24 @@ class PyPi(AbstractRecipeModel):
         :param sdist_metadata: sdist metadata
         :return: list with all entry points
         """
-        all_entry_points = sdist_metadata.get("entry_points", None)
-        if all_entry_points and (
-            all_entry_points.get("console_scripts")
-            or all_entry_points.get("gui_scripts")
+        all_entry_points = sdist_metadata.get("entry_points", {})
+        if isinstance(all_entry_points, str) or not all_entry_points:
+            return []
+        if all_entry_points.get("console_scripts") or all_entry_points.get(
+            "gui_scripts"
         ):
-            return all_entry_points.get("console_scripts", []) + all_entry_points.get(
-                "gui_scripts", []
-            )
+            console_scripts = all_entry_points.get("console_scripts", [])
+            gui_scripts = all_entry_points.get("gui_scripts", [])
+            entry_points_result = []
+            if console_scripts:
+                if isinstance(console_scripts, str):
+                    console_scripts = [console_scripts]
+                entry_points_result += console_scripts
+            if gui_scripts:
+                if isinstance(gui_scripts, str):
+                    gui_scripts = [gui_scripts]
+                entry_points_result += gui_scripts
+            return entry_points_result
         return []
 
     @staticmethod
@@ -385,23 +451,46 @@ class PyPi(AbstractRecipeModel):
         :param sdist_metadata: sdist metadata
         :return: list with all requirements
         """
-        pypi_deps_name = set()
-        requires_dist = []
         all_deps = []
-        if pypi_metadata.get("requires_dist"):
-            all_deps = pypi_metadata.get("requires_dist", [])
         if sdist_metadata.get("install_requires"):
             all_deps += sdist_metadata.get("install_requires", [])
+        if pypi_metadata.get("requires_dist"):
+            all_deps += pypi_metadata.get("requires_dist", [])
 
-        for sdist_pkg in all_deps:
-            match_deps = PyPi.RE_DEPS_NAME.match(sdist_pkg)
-            if match_deps:
+        re_search = re.compile(r";\s*extra")
+        all_deps = [pkg for pkg in all_deps if not re_search.search(pkg)]
+        current_pkg = pypi_metadata.get("name", "")
+
+        requires_dist = []
+        pypi_deps_name = set()
+        with progressbar_with_status(len(all_deps)) as bar:
+            for pos, sdist_pkg in enumerate(all_deps, 1):
+                match_deps = PyPi.RE_DEPS_NAME.match(sdist_pkg)
+                if not match_deps:
+                    bar.update(pos)
+                    continue
                 match_deps = match_deps.group(0).strip()
-                if match_deps not in pypi_deps_name:
-                    pypi_deps_name.add(match_deps)
-                    requires_dist.append(sdist_pkg)
+                pkg_name = PyPi._normalize_pkg_name(match_deps)
+                bar.update(pos, pkg_name=pkg_name)
+                if current_pkg and current_pkg == pkg_name:
+                    continue
+                if pkg_name in pypi_deps_name:
+                    continue
+
+                pypi_deps_name.add(pkg_name)
+                requires_dist.append(sdist_pkg.replace(match_deps, pkg_name))
         return requires_dist
 
+    @staticmethod
+    def _normalize_pkg_name(pkg_name: str) -> str:
+        if is_pkg_available(pkg_name):
+            return pkg_name
+        if is_pkg_available(pkg_name.replace("-", "_")):
+            return pkg_name.replace("-", "_")
+        elif is_pkg_available(pkg_name.replace("_", "-")):
+            return pkg_name.replace("_", "-")
+        return pkg_name
+
     def refresh_section(self, section: str = ""):
         """Update one specific section.
 
@@ -439,40 +528,39 @@ class PyPi(AbstractRecipeModel):
         if license_metadata:
             license_name = license_metadata.name
             if license_metadata.path:
-                license_file = "LICENSE"
                 if license_metadata.is_packaged:
                     license_file = license_metadata.path
                 else:
+                    license_file = os.path.basename(license_metadata.path)
                     self.files_to_copy.append(license_metadata.path)
 
-        print(f"{Fore.LIGHTBLACK_EX}License type: {Fore.LIGHTMAGENTA_EX}{license_name}")
-        print(f"{Fore.LIGHTBLACK_EX}License file: {Fore.LIGHTMAGENTA_EX}{license_file}")
+        print_msg(f"License type: {Fore.LIGHTMAGENTA_EX}{license_name}")
+        print_msg(f"License file: {Fore.LIGHTMAGENTA_EX}{license_file}")
 
         all_requirements = self._extract_requirements(metadata)
+        all_requirements["host"] = solve_list_pkg_name(
+            all_requirements["host"], self.PYPI_CONFIG
+        )
+        all_requirements["run"] = solve_list_pkg_name(
+            all_requirements["run"], self.PYPI_CONFIG
+        )
+        print_requirements(all_requirements)
 
-        def print_req(name, list_req: List):
-            prefix_req = f"{Fore.LIGHTBLACK_EX}\n   - {Fore.LIGHTCYAN_EX}"
-            print(
-                f"{Fore.LIGHTBLACK_EX}{name} requirements:"
-                f"{prefix_req}{prefix_req.join(list_req)}"
-            )
-
-        if all_requirements.get("build"):
-            print_req("Build", all_requirements["build"])
-        print_req("Host", all_requirements["host"])
-        print_req("run", all_requirements["run"])
-
+        test_entry_points = PyPi._get_test_entry_points(metadata.get("entry_points"))
+        test_imports = PyPi._get_test_imports(metadata, pypi_metadata["name"])
         return {
             "package": {"name": name, "version": metadata["version"]},
             "build": {"entry_points": metadata.get("entry_points")},
             "requirements": all_requirements,
             "test": {
-                "imports": pypi_metadata["name"].replace("-", "_"),
-                "commands": "pip check",
+                "imports": test_imports,
+                "commands": ["pip check"] + test_entry_points,
                 "requires": "pip",
             },
             "about": {
-                "home": metadata.get("project_url"),
+                "home": metadata["url"]
+                if metadata.get("url")
+                else metadata.get("project_url"),
                 "summary": metadata.get("summary"),
                 "doc_url": metadata.get("doc_url"),
                 "dev_url": metadata.get("dev_url"),
@@ -482,6 +570,27 @@ class PyPi(AbstractRecipeModel):
             "source": metadata.get("source", {}),
         }
 
+    @staticmethod
+    def _get_test_imports(metadata: dict, default: Optional[str] = None) -> List:
+        if default:
+            default = default.replace("-", "_")
+        if "packages" not in metadata or not metadata["packages"]:
+            return [default]
+        meta_pkg = metadata["packages"]
+        if isinstance(meta_pkg, str):
+            meta_pkg = [metadata["packages"]]
+        return sorted(meta_pkg)[:2]
+
+    @staticmethod
+    def _get_test_entry_points(entry_points: Union[List, str]) -> List:
+        if entry_points:
+            if isinstance(entry_points, str):
+                entry_points = [entry_points]
+        test_entry_points = [
+            f"{ep.split('=')[0].strip()} --help" for ep in entry_points
+        ]
+        return test_entry_points
+
     @staticmethod
     def _discover_license(metadata: dict) -> Optional[ShortLicense]:
         """Based on the metadata this method will try to discover what is the
@@ -492,8 +601,11 @@ class PyPi(AbstractRecipeModel):
         the license.
         """
         git_url = metadata.get("dev_url", None)
-        if not git_url and "github.com/" in metadata.get("project_url"):
+        if not git_url and "github.com/" in metadata.get("project_url", ""):
             git_url = metadata.get("project_url")
+        # "url" is always present but sometimes set to None
+        if not git_url and "github.com/" in (metadata.get("url") or ""):
+            git_url = metadata.get("url")
 
         short_license = search_license_file(
             metadata.get("sdist_path"),
@@ -504,9 +616,8 @@ class PyPi(AbstractRecipeModel):
         if short_license:
             return short_license
 
-    @staticmethod
     @lru_cache(maxsize=10)
-    def _get_pypi_metadata(name, version: Optional[str] = None) -> dict:
+    def _get_pypi_metadata(self, name, version: Optional[str] = None) -> dict:
         """Method responsible to communicate with the pypi api endpoints and
         get the whole metadata available for the specified package and version.
 
@@ -514,7 +625,7 @@ class PyPi(AbstractRecipeModel):
         :param version: Package version
         :return: Pypi metadata
         """
-        print(f"{Fore.LIGHTBLACK_EX}Recovering metadata from pypi...")
+        print_msg("Recovering metadata from pypi...")
         if version:
             url_pypi = PyPi.URL_PYPI_METADATA.format(pkg_name=f"{name}/{version}")
         else:
@@ -529,6 +640,13 @@ class PyPi(AbstractRecipeModel):
             )
 
         metadata = metadata.json()
+        if self._download:
+            download_file = os.path.join(
+                str(mkdtemp(f"grayskull-pypi-metadata-{name}-")), "pypi.json"
+            )
+            with open(download_file, "w") as f:
+                json.dump(metadata, f, indent=4)
+            self.files_to_copy.append(download_file)
         info = metadata["info"]
         project_urls = info.get("project_urls") if info.get("project_urls") else {}
         log.info(f"Package: {name}=={info['version']}")
@@ -542,6 +660,7 @@ class PyPi(AbstractRecipeModel):
             "project_url": info.get("project_url"),
             "doc_url": info.get("docs_url"),
             "dev_url": project_urls.get("Source"),
+            "url": info.get("home_page"),
             "license": info.get("license"),
             "source": {
                 "url": "https://pypi.io/packages/source/{{ name[0] }}/{{ name }}/"
@@ -601,7 +720,7 @@ class PyPi(AbstractRecipeModel):
         )
         host_req = PyPi._format_dependencies(setup_requires, name)
 
-        if not requires_dist and not host_req:
+        if not requires_dist and not host_req and not metadata.get("requires_python"):
             return {"host": sorted(["python", "pip"]), "run": ["python"]}
 
         run_req = self._get_run_req_from_requires_dist(requires_dist)
@@ -625,13 +744,22 @@ class PyPi(AbstractRecipeModel):
             host_req += [f"python{limit_python}", "pip"]
 
         run_req.insert(0, f"python{limit_python}")
-        result = (
-            {"build": sorted(map(lambda x: x.lower(), build_req))} if build_req else {}
-        )
+        result = {}
+        if build_req:
+            result = {
+                "build": PyPi.__rm_duplicated_deps(
+                    sorted(map(lambda x: x.lower(), build_req))
+                )
+            }
+
         result.update(
             {
-                "host": sorted(map(lambda x: x.lower(), host_req)),
-                "run": sorted(map(lambda x: x.lower(), run_req)),
+                "host": PyPi.__rm_duplicated_deps(
+                    sorted(map(lambda x: x.lower(), host_req))
+                ),
+                "run": PyPi.__rm_duplicated_deps(
+                    sorted(map(lambda x: x.lower(), run_req))
+                ),
             }
         )
         self._update_requirements_with_pin(result)
@@ -646,12 +774,14 @@ class PyPi(AbstractRecipeModel):
         :param name: package name
         :return: list of dependencies formatted
         """
-        formated_dependencies = []
+        formatted_dependencies = []
         re_deps = re.compile(
             r"^\s*([\.a-zA-Z0-9_-]+)\s*(.*)\s*$", re.MULTILINE | re.DOTALL
         )
         re_remove_space = re.compile(r"([<>!=]+)\s+")
         re_remove_tags = re.compile(r"\s*(\[.*\])", re.DOTALL)
+        re_remove_comments = re.compile(r"\s+#.*", re.DOTALL)
+
         for req in all_dependencies:
             match_req = re_deps.match(req)
             deps_name = req
@@ -664,8 +794,9 @@ class PyPi(AbstractRecipeModel):
                     deps_name = " ".join(match_req)
             deps_name = re_remove_space.sub(r"\1", deps_name.strip())
             deps_name = re_remove_tags.sub(r"", deps_name.strip())
-            formated_dependencies.append(deps_name)
-        return formated_dependencies
+            deps_name = re_remove_comments.sub("", deps_name)
+            formatted_dependencies.append(deps_name.strip())
+        return formatted_dependencies
 
     @staticmethod
     def _update_requirements_with_pin(requirements: dict):
@@ -746,7 +877,7 @@ class PyPi(AbstractRecipeModel):
                     result_selector.append(extra[4])
                 if extra[5]:
                     result_selector.append(extra[5])
-        if result_selector[-1] in ["and", "or"]:
+        if result_selector and result_selector[-1] in ["and", "or"]:
             del result_selector[-1]
         return result_selector
 
@@ -800,6 +931,7 @@ class PyPi(AbstractRecipeModel):
             return None
 
         py_ver_enabled = PyPi._get_py_version_available(req_python)
+        small_py3_version = get_small_py3_version(list(py_ver_enabled.keys()))
         all_py = list(py_ver_enabled.values())
         if all(all_py):
             return None
@@ -807,12 +939,14 @@ class PyPi(AbstractRecipeModel):
             return (
                 "# [py2k]"
                 if is_selector
-                else f">={SUPPORTED_PY[1].major}.{SUPPORTED_PY[1].minor}"
+                else f">={small_py3_version.major}.{small_py3_version.minor}"
             )
         if py_ver_enabled.get(PyVer(2, 7)) and any(all_py[1:]) is False:
             return "# [py3k]" if is_selector else "<3.0"
 
-        for pos, py_ver in enumerate(SUPPORTED_PY[1:], 1):
+        for pos, py_ver in enumerate(py_ver_enabled):
+            if py_ver == PyVer(2, 7):
+                continue
             if all(all_py[pos:]) and any(all_py[:pos]) is False:
                 return (
                     f"# [py<{py_ver.major}{py_ver.minor}]"
@@ -828,7 +962,7 @@ class PyPi(AbstractRecipeModel):
                 else:
                     py2 = ""
                     if not all_py[0]:
-                        py2 = f">=3.6,"
+                        py2 = f">={small_py3_version.major}.{small_py3_version.minor},"
                     return f"{py2}<{py_ver.major}.{py_ver.minor}"
 
         all_selector = PyPi._get_py_multiple_selectors(
@@ -859,12 +993,21 @@ class PyPi(AbstractRecipeModel):
         :param req_python: Requires python
         :return: Dict of Python versions if it is enabled or disabled
         """
-        py_ver_enabled = {py_ver: True for py_ver in SUPPORTED_PY}
+        sup_python_ver = deepcopy(SUPPORTED_PY)
+        for _, major, minor in req_python:
+            if not minor:
+                minor = 0
+            new_py_ver = PyVer(int(major), int(minor))
+            if new_py_ver in sup_python_ver:
+                continue
+            sup_python_ver.append(new_py_ver)
+        sup_python_ver.sort()
+        py_ver_enabled = {py_ver: True for py_ver in sup_python_ver}
         for op, major, minor in req_python:
             if not minor:
                 minor = 0
-            for sup_py in SUPPORTED_PY:
-                if py_ver_enabled[sup_py] is False:
+            for sup_py, is_enabled in py_ver_enabled.items():
+                if is_enabled is False:
                     continue
                 py_ver_enabled[sup_py] = eval(
                     f"sup_py {op} PyVer(int({major}), int({minor}))"
@@ -883,7 +1026,11 @@ class PyPi(AbstractRecipeModel):
         """
         all_selector = []
         if selectors[PyVer(2, 7)] is False:
-            all_selector += ["py2k"] if is_selector else [">=3.6"]
+            all_selector += (
+                ["py2k"]
+                if is_selector
+                else get_small_py3_version(list(selectors.keys()))
+            )
         for py_ver, is_enabled in selectors.items():
             if py_ver == PyVer(2, 7) or is_enabled:
                 continue
@@ -913,4 +1060,33 @@ class PyPi(AbstractRecipeModel):
             return f"py{operation}{value}"
         if option == "sys_platform":
             value = re.sub(r"[^a-zA-Z]+", "", value)
+            if operation == "!=":
+                return f"not {value.lower()}"
             return value.lower()
+        if option == "platform_system":
+            replace_val = {"windows": "win", "linux": "linux", "darwin": "osx"}
+            value_lower = value.lower().strip()
+            if value_lower in replace_val:
+                value_lower = replace_val[value_lower]
+            if operation == "!=":
+                return f"not {value_lower}"
+            return value_lower
+
+
+def get_small_py3_version(list_py_ver: List[PyVer]) -> PyVer:
+    list_py_ver = sorted(list_py_ver)
+    for py_ver in list_py_ver:
+        if py_ver >= PyVer(3, 0):
+            return py_ver
+
+
+def search_setup_root(path_folder: Union[Path, str]) -> Path:
+    setup_py = list(Path(path_folder).rglob("setup.py"))
+    if setup_py:
+        return setup_py[0]
+    setup_cfg = list(Path(path_folder).rglob("setup.cfg"))
+    if setup_cfg:
+        return setup_cfg[0]
+    pyproject_toml = list(Path(path_folder).rglob("pyproject.toml"))
+    if pyproject_toml:
+        return pyproject_toml[0]
